@@ -4,46 +4,52 @@
  * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
-#include "_debug.h"
 #include "application.h"
-#include "haction.h"
-#include "lockmanager.h"
-#include "mpvobject.h"
 #include "audiosettings.h"
+#include "customcommandsmodel.h"
 #include "generalsettings.h"
-#include "mousesettings.h"
+#include "global.h"
+#include "haction.h"
+#include "haruna-version.h"
+#include "lockmanager.h"
 #include "mediaplayer2.h"
 #include "mediaplayer2player.h"
+#include "mousesettings.h"
+#include "mpvobject.h"
 #include "playbacksettings.h"
-#include "playlistsettings.h"
-#include "subtitlessettings.h"
-#include "videosettings.h"
 #include "playlistitem.h"
 #include "playlistmodel.h"
+#include "playlistsettings.h"
 #include "subtitlesfoldersmodel.h"
+#include "subtitlessettings.h"
 #include "thumbnailimageprovider.h"
 #include "tracksmodel.h"
+#include "videosettings.h"
 #include "worker.h"
 
+#include <clocale>
+
 #include <QApplication>
 #include <QCommandLineParser>
 #include <QCoreApplication>
-#include <QGuiApplication>
+#include <QDesktopServices>
 #include <QDir>
 #include <QDBusConnection>
 #include <QFileInfo>
+#include <QGuiApplication>
+#include <QMimeDatabase>
 #include <QPointer>
 #include <QQmlApplicationEngine>
 #include <QQmlContext>
+#include <QQmlEngine>
 #include <QQuickItem>
 #include <QQuickStyle>
 #include <QQuickView>
 #include <QStandardPaths>
-#include <QStyleFactory>
 #include <QStyle>
+#include <QStyleFactory>
 #include <QThread>
-#include <QQmlEngine>
-#include <QMimeDatabase>
+#include <KFileItem>
 
 #include <KAboutApplicationDialog>
 #include <KAboutData>
@@ -51,20 +57,20 @@
 #include <KConfig>
 #include <KConfigGroup>
 #include <KFileMetaData/Properties>
-#include <KI18n/KLocalizedString>
+#include <KLocalizedContext>
 #include <KLocalizedString>
 #include <KShortcutsDialog>
+#include <KTreeWidgetSearchLine>
 
 static QApplication *createApplication(int &argc, char **argv, const QString &applicationName)
 {
     QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
     QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
-    QApplication::setOrganizationName("georgefb");
+    QApplication::setOrganizationName("kde");
     QApplication::setApplicationName(applicationName);
-    QApplication::setOrganizationDomain("georgefb.com");
+    QApplication::setOrganizationDomain("kde.org");
     QApplication::setApplicationDisplayName("Haruna - Video Player");
     QApplication::setApplicationVersion(Application::version());
-    QApplication::setWindowIcon(QIcon::fromTheme("org.kde.haruna"));
 
     QQuickStyle::setStyle(QStringLiteral("org.kde.desktop"));
     QQuickStyle::setFallbackStyle(QStringLiteral("Fusion"));
@@ -73,6 +79,8 @@ static QApplication *createApplication(int &argc, char **argv, const QString &ap
     }
 
     QApplication *app = new QApplication(argc, argv);
+    QApplication::setWindowIcon(QIcon::fromTheme("haruna"));
+    KLocalizedString::setApplicationDomain("haruna");
     return app;
 }
 
@@ -80,11 +88,21 @@ Application::Application(int &argc, char **argv, const QString &applicationName)
     : m_app(createApplication(argc, argv, applicationName))
     , m_collection(this)
 {
-    m_config = KSharedConfig::openConfig("georgefb/haruna.conf");
+    m_config = KSharedConfig::openConfig(Global::instance()->appConfigFilePath());
     m_shortcuts = new KConfigGroup(m_config, "Shortcuts");
     m_schemes = new KColorSchemeManager(this);
     m_systemDefaultStyle = m_app->style()->objectName();
 
+    // used to hide playlist when mouse leaves the application
+    // while moving between monitors while in fullscreen
+    auto *appEventFilter = new ApplicationEventFilter();
+    m_app->installEventFilter(appEventFilter);
+    QObject::connect(appEventFilter, &ApplicationEventFilter::applicationMouseLeave,
+                     this, &Application::qmlApplicationMouseLeave);
+
+
+    setupUserActions();
+
     // register mpris dbus service
     QString mspris2Name(QStringLiteral("org.mpris.MediaPlayer2.haruna"));
     QDBusConnection::sessionBus().registerService(mspris2Name);
@@ -151,8 +169,9 @@ void Application::setupAboutData()
     m_aboutData.setShortDescription(i18n("A configurable video player."));
     m_aboutData.setLicense(KAboutLicense::GPL_V3);
     m_aboutData.setCopyrightStatement(i18n("(c) 2019-2021"));
-    m_aboutData.setHomepage(QStringLiteral("https://github.com/g-fb/haruna"));
-    m_aboutData.setBugAddress(QStringLiteral("https://github.com/g-fb/haruna/issues").toUtf8());
+    m_aboutData.setHomepage(QStringLiteral("https://invent.kde.org/multimedia/haruna"));
+    m_aboutData.setBugAddress(QStringLiteral("https://bugs.kde.org/enter_bug.cgi?product=Haruna").toUtf8());
+    m_aboutData.setComponentName(QStringLiteral("generic"));
     m_aboutData.setDesktopFileName("org.kde.haruna");
 
     m_aboutData.addAuthor(i18n("George Florea Bănuș"),
@@ -226,38 +245,64 @@ void Application::setupQmlContextProperties()
                                             QStringLiteral("LockManager should not be created in QML"));
 
     m_engine->rootContext()->setContextProperty(QStringLiteral("subsFoldersModel"), subsFoldersModel.release());
+
+    auto customCommandsModel = new CustomCommandsModel();
+    m_engine->rootContext()->setContextProperty(QStringLiteral("customCommandsModel"), customCommandsModel);
+    auto proxyCustomCommandsModel = new ProxyCustomCommandsModel();
+    proxyCustomCommandsModel->setSourceModel(customCommandsModel);
+    m_engine->rootContext()->setContextProperty(QStringLiteral("proxyCustomCommandsModel"), proxyCustomCommandsModel);
+
+    m_engine->rootContext()->setContextObject(new KLocalizedContext(this));
+    m_engine->rootContext()->setContextProperty(QStringLiteral("harunaAboutData"),
+                                                QVariant::fromValue(KAboutData::applicationData()));
 }
 
 QUrl Application::configFilePath()
 {
-    auto configPath = QStandardPaths::writableLocation(m_config->locationType());
-    auto configFilePath = configPath.append(QStringLiteral("/")).append(m_config->name());
-    QUrl url(configFilePath);
+    QUrl url(Global::instance()->appConfigFilePath());
     url.setScheme("file");
     return url;
 }
 
 QUrl Application::configFolderPath()
 {
-    auto configPath = QStandardPaths::writableLocation(m_config->locationType());
-    auto configFilePath = configPath.append(QStringLiteral("/")).append(m_config->name());
-    QFileInfo fileInfo(configFilePath);
-    QUrl url(fileInfo.absolutePath());
+    QUrl url(Global::instance()->appConfigDirPath());
     url.setScheme("file");
     return url;
 }
 
+bool Application::configFolderExists()
+{
+    QFileInfo fi(Global::instance()->appConfigDirPath());
+    return fi.exists();
+}
+
 QString Application::version()
 {
-    return QStringLiteral("0.6.3");
+    return QStringLiteral(HARUNA_VERSION_STRING);
 }
 
 bool Application::hasYoutubeDl()
 {
-    return !QStandardPaths::findExecutable(QStringLiteral("youtube-dl")).isEmpty();
+    return !youtubeDlExecutable().isEmpty();
 
 }
 
+QString Application::youtubeDlExecutable()
+{
+    auto ytDlp = QStandardPaths::findExecutable(QStringLiteral("yt-dlp"));
+    if (!ytDlp.isEmpty()) {
+        return ytDlp;
+    }
+
+    auto youtubeDl = QStandardPaths::findExecutable(QStringLiteral("youtube-dl"));
+    if (!youtubeDl.isEmpty()) {
+        return youtubeDl;
+    }
+
+    return QString();
+}
+
 QUrl Application::parentUrl(const QString &path)
 {
     QUrl url(path);
@@ -343,8 +388,13 @@ QString Application::getFileContent(const QString &file)
 QString Application::mimeType(const QString &file)
 {
     QMimeDatabase db;
-    QMimeType mime = db.mimeTypeForFile(file);
-    return mime.name();
+    QMimeType mimeType;
+    if(KFileItem(file).isSlow()) {
+        mimeType = db.mimeTypeForFile(file, QMimeDatabase::MatchExtension);
+    } else {
+        mimeType = db.mimeTypeForFile(file);
+    }
+    return mimeType.name();
 }
 
 QStringList Application::availableGuiStyles()
@@ -371,13 +421,21 @@ void Application::activateColorScheme(const QString &name)
     m_schemes->activateScheme(m_schemes->indexForScheme(name));
 }
 
-void Application::configureShortcuts()
+void Application::configureShortcuts(const QString &name)
 {
     KShortcutsDialog dlg(KShortcutsEditor::ApplicationAction, KShortcutsEditor::LetterShortcutsAllowed, nullptr);
     connect(&dlg, &KShortcutsDialog::accepted, this, [ = ](){
         m_collection.writeSettings(m_shortcuts);
         m_config->sync();
     });
+
+    if (!name.isEmpty()) {
+        auto searchLine = dlg.findChild<KTreeWidgetSearchLine *>();
+        if (searchLine != nullptr) {
+            searchLine->setText(name);
+        }
+    }
+
     dlg.setModal(true);
     dlg.addCollection(&m_collection);
     dlg.configure(false);
@@ -414,7 +472,7 @@ void Application::setupActions(const QString &actionName)
         auto action = new HAction();
         action->setText(i18n("Configure Keyboard Shortcuts"));
         action->setIcon(QIcon::fromTheme("configure-shortcuts"));
-        connect(action, &QAction::triggered, this, &Application::configureShortcuts);
+        connect(action, &QAction::triggered, this, [=]() { configureShortcuts(); });
         m_collection.setDefaultShortcut(action, Qt::CTRL + Qt::SHIFT + Qt::Key_S);
         m_collection.addAction(actionName, action);
     }
@@ -453,7 +511,7 @@ void Application::setupActions(const QString &actionName)
     if (actionName == QStringLiteral("openFile")) {
         auto action = new HAction();
         action->setText(i18n("Open File"));
-        action->setIcon(QIcon::fromTheme("folder-videos-symbolic"));
+        action->setIcon(QIcon::fromTheme("folder-videos"));
         m_collection.setDefaultShortcut(action, Qt::CTRL + Qt::Key_O);
         m_collection.addAction(actionName, action);
     }
@@ -469,10 +527,9 @@ void Application::setupActions(const QString &actionName)
     if (actionName == QStringLiteral("aboutHaruna")) {
         auto action = new HAction();
         action->setText(i18n("About Haruna"));
-        action->setIcon(QIcon::fromTheme("help-about-symbolic"));
+        action->setIcon(QIcon::fromTheme("help-about"));
         m_collection.setDefaultShortcut(action, Qt::Key_F1);
         m_collection.addAction(actionName, action);
-        connect(action, &QAction::triggered, this, &Application::aboutApplication);
     }
 
     // mpv actions
@@ -831,5 +888,49 @@ void Application::setupActions(const QString &actionName)
         m_collection.setDefaultShortcut(action, Qt::Key_D);
         m_collection.addAction(actionName, action);
     }
+    if (actionName == QStringLiteral("exitFullscreen")) {
+        auto action = new HAction();
+        action->setText(i18n("Exit Fullscreen"));
+        m_collection.setDefaultShortcut(action, Qt::Key_Escape);
+        m_collection.addAction(actionName, action);
+    }
+    m_collection.readSettings(m_shortcuts);
+}
+
+void Application::createUserAction(const QString &text)
+{
+    KSharedConfig::Ptr m_customCommandsConfig;
+    QString ccConfig = Global::instance()->appConfigFilePath(Global::ConfigFile::CustomCommands);
+    m_customCommandsConfig = KSharedConfig::openConfig(ccConfig, KConfig::SimpleConfig);
+    int counter = m_customCommandsConfig->group(QString()).readEntry("Counter", 0);
+    const QString &name = QString("Command_%1").arg(counter);
+
+    auto action = new HAction();
+    action->setText(text);
+    m_collection.addAction(name, action);
+    m_collection.readSettings(m_shortcuts);
+}
+
+void Application::openDocs(const QString &page)
+{
+    QDesktopServices::openUrl(QUrl(page));
+}
+
+void Application::setupUserActions()
+{
+    KSharedConfig::Ptr m_customCommandsConfig;
+    QString ccConfig = Global::instance()->appConfigFilePath(Global::ConfigFile::CustomCommands);
+    m_customCommandsConfig = KSharedConfig::openConfig(ccConfig, KConfig::SimpleConfig);
+    QStringList groups = m_customCommandsConfig->groupList();
+    for (const QString &_group : qAsConst((groups))) {
+        auto configGroup = m_customCommandsConfig->group(_group);
+        QString command = configGroup.readEntry("Command", QString());
+
+        if (configGroup.readEntry("Type", QString()) == "shortcut") {
+            auto action = new HAction();
+            action->setText(command);
+            m_collection.addAction(_group, action);
+        }
+    }
     m_collection.readSettings(m_shortcuts);
 }
