@@ -22,6 +22,25 @@ class KConfigDialog;
 class KColorSchemeManager;
 class QAction;
 
+class ApplicationEventFilter : public QObject
+{
+    Q_OBJECT
+
+signals:
+    void applicationMouseLeave();
+
+protected:
+    bool eventFilter(QObject *obj, QEvent *event) {
+        if (event->type() == QEvent::Leave) {
+            Q_EMIT applicationMouseLeave();
+            return true;
+        } else {
+            // standard event processing
+            return QObject::eventFilter(obj, event);
+        }
+    }
+};
+
 class Application : public QObject
 {
     Q_OBJECT
@@ -36,24 +55,31 @@ public:
     int run();
     QUrl configFilePath();
     QUrl configFolderPath();
+    Q_INVOKABLE bool configFolderExists();
     Q_INVOKABLE QUrl parentUrl(const QString &path);
     Q_INVOKABLE QUrl pathToUrl(const QString &path);
     Q_INVOKABLE QString argument(int key);
     Q_INVOKABLE void addArgument(int key, const QString &value);
     Q_INVOKABLE QAction *action(const QString &name);
     Q_INVOKABLE QString getFileContent(const QString &file);
-    Q_INVOKABLE QString mimeType(const QString &file);
     Q_INVOKABLE QStringList availableGuiStyles();
     Q_INVOKABLE void setGuiStyle(const QString &style);
     Q_INVOKABLE void activateColorScheme(const QString &name);
-    Q_INVOKABLE void configureShortcuts();
+    Q_INVOKABLE void configureShortcuts(const QString &name = QString());
+    Q_INVOKABLE void createUserAction(const QString &text);
+    Q_INVOKABLE void openDocs(const QString &page);
 
     static QString version();
     Q_INVOKABLE static bool hasYoutubeDl();
+    Q_INVOKABLE static QString youtubeDlExecutable();
     Q_INVOKABLE static bool isYoutubePlaylist(const QString &path);
     Q_INVOKABLE static QString formatTime(const double time);
     Q_INVOKABLE static void hideCursor();
     Q_INVOKABLE static void showCursor();
+    Q_INVOKABLE static QString mimeType(const QString &file);
+
+signals:
+    void qmlApplicationMouseLeave();
 
 private:
     void setupWorkerThread();
@@ -64,6 +90,7 @@ private:
     void setupQmlContextProperties();
     void aboutApplication();
     void setupActions(const QString &actionName);
+    void setupUserActions();
     QAbstractItemModel *colorSchemesModel();
     QApplication *m_app;
     QQmlApplicationEngine *m_engine;
