@@ -4,7 +4,6 @@
  * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
-#include "_debug.h"
 #include "mpvobject.h"
 #include "application.h"
 #include "generalsettings.h"
@@ -13,6 +12,7 @@
 #include "playlistitem.h"
 #include "track.h"
 #include "tracksmodel.h"
+#include "global.h"
 
 #include <QCryptographicHash>
 #include <QDir>
@@ -27,6 +27,8 @@
 #include <QStandardPaths>
 #include <QtGlobal>
 
+#include <KShell>
+
 void on_mpv_redraw(void *ctx)
 {
     QMetaObject::invokeMethod(static_cast<MpvObject*>(ctx), "update", Qt::QueuedConnection);
@@ -87,7 +89,7 @@ QOpenGLFramebufferObject * MpvRenderer::createFramebufferObject(const QSize &siz
         if (mpv_render_context_create(&obj->mpv_gl, obj->mpv, params) < 0)
             throw std::runtime_error("failed to initialize mpv GL context");
         mpv_render_context_set_update_callback(obj->mpv_gl, on_mpv_redraw, obj);
-        emit obj->ready();
+        Q_EMIT obj->ready();
     }
 
     return QQuickFramebufferObject::Renderer::createFramebufferObject(size);
@@ -112,6 +114,8 @@ MpvObject::MpvObject(QQuickItem * parent)
     setProperty("screenshot-template", VideoSettings::screenshotTemplate());
     setProperty("sub-auto", "exact");
     setProperty("volume-max", "100");
+    // set ytdl_path to yt-dlp or fallback to youtube-dl
+    setProperty("script-opts", QString("ytdl_hook-ytdl_path=%1").arg(Application::youtubeDlExecutable()));
 
     mpv_observe_property(mpv, 0, "media-title", MPV_FORMAT_STRING);
     mpv_observe_property(mpv, 0, "time-pos", MPV_FORMAT_DOUBLE);
@@ -132,6 +136,20 @@ MpvObject::MpvObject(QQuickItem * parent)
     if (mpv_initialize(mpv) < 0)
         throw std::runtime_error("could not initialize mpv context");
 
+
+    // run user commands
+    KSharedConfig::Ptr m_customPropsConfig;
+    QString ccConfig = Global::instance()->appConfigFilePath(Global::ConfigFile::CustomCommands);
+    m_customPropsConfig = KSharedConfig::openConfig(ccConfig, KConfig::SimpleConfig);
+    QStringList groups = m_customPropsConfig->groupList();
+    for (const QString &_group : qAsConst((groups))) {
+        auto configGroup = m_customPropsConfig->group(_group);
+        QString type = configGroup.readEntry("Type", QString());
+        if (type == "startup") {
+            userCommand(configGroup.readEntry("Command", QString()));
+        }
+    }
+
     mpv_set_wakeup_callback(mpv, MpvObject::mpvEvents, this);
 
     connect(this, &MpvObject::fileLoaded,
@@ -181,7 +199,7 @@ void MpvObject::setPosition(double value)
         return;
     }
     setProperty("time-pos", value);
-    emit positionChanged();
+    Q_EMIT positionChanged();
 }
 
 double MpvObject::remaining()
@@ -205,7 +223,7 @@ void MpvObject::setPause(bool value)
         return;
     }
     setProperty("pause", value);
-    emit pauseChanged();
+    Q_EMIT pauseChanged();
 }
 
 int MpvObject::volume()
@@ -219,7 +237,7 @@ void MpvObject::setVolume(int value)
         return;
     }
     setProperty("volume", value);
-    emit volumeChanged();
+    Q_EMIT volumeChanged();
 }
 
 int MpvObject::chapter()
@@ -233,7 +251,7 @@ void MpvObject::setChapter(int value)
         return;
     }
     setProperty("chapter", value);
-    emit chapterChanged();
+    Q_EMIT chapterChanged();
 }
 
 int MpvObject::audioId()
@@ -247,7 +265,7 @@ void MpvObject::setAudioId(int value)
         return;
     }
     setProperty("aid", value);
-    emit audioIdChanged();
+    Q_EMIT audioIdChanged();
 }
 
 int MpvObject::subtitleId()
@@ -261,7 +279,7 @@ void MpvObject::setSubtitleId(int value)
         return;
     }
     setProperty("sid", value);
-    emit subtitleIdChanged();
+    Q_EMIT subtitleIdChanged();
 }
 
 int MpvObject::secondarySubtitleId()
@@ -275,7 +293,7 @@ void MpvObject::setSecondarySubtitleId(int value)
         return;
     }
     setProperty("secondary-sid", value);
-    emit secondarySubtitleIdChanged();
+    Q_EMIT secondarySubtitleIdChanged();
 }
 
 int MpvObject::contrast()
@@ -289,7 +307,7 @@ void MpvObject::setContrast(int value)
         return;
     }
     setProperty("contrast", value);
-    emit contrastChanged();
+    Q_EMIT contrastChanged();
 }
 
 int MpvObject::brightness()
@@ -303,7 +321,7 @@ void MpvObject::setBrightness(int value)
         return;
     }
     setProperty("brightness", value);
-    emit brightnessChanged();
+    Q_EMIT brightnessChanged();
 }
 
 int MpvObject::gamma()
@@ -317,7 +335,7 @@ void MpvObject::setGamma(int value)
         return;
     }
     setProperty("gamma", value);
-    emit gammaChanged();
+    Q_EMIT gammaChanged();
 }
 
 int MpvObject::saturation()
@@ -331,7 +349,7 @@ void MpvObject::setSaturation(int value)
         return;
     }
     setProperty("saturation", value);
-    emit saturationChanged();
+    Q_EMIT saturationChanged();
 }
 
 double MpvObject::watchPercentage()
@@ -345,7 +363,7 @@ void MpvObject::setWatchPercentage(double value)
         return;
     }
     m_watchPercentage = value;
-    emit watchPercentageChanged();
+    Q_EMIT watchPercentageChanged();
 }
 
 bool MpvObject::hwDecoding()
@@ -364,7 +382,7 @@ void MpvObject::setHWDecoding(bool value)
     } else  {
         setProperty("hwdec", "no");
     }
-    emit hwDecodingChanged();
+    Q_EMIT hwDecodingChanged();
 }
 
 QQuickFramebufferObject::Renderer *MpvObject::createRenderer() const
@@ -402,19 +420,19 @@ void MpvObject::eventHandler()
         }
         switch (event->event_id) {
         case MPV_EVENT_START_FILE: {
-            emit fileStarted();
+            Q_EMIT fileStarted();
             break;
         }
         case MPV_EVENT_FILE_LOADED: {
-            emit fileLoaded();
+            Q_EMIT fileLoaded();
             break;
         }
         case MPV_EVENT_END_FILE: {
             auto prop = (mpv_event_end_file *)event->data;
             if (prop->reason == MPV_END_FILE_REASON_EOF) {
-                emit endFile("eof");
+                Q_EMIT endFile("eof");
             } else if(prop->reason == MPV_END_FILE_REASON_ERROR) {
-                emit endFile("error");
+                Q_EMIT endFile("error");
             }
             break;
         }
@@ -423,59 +441,59 @@ void MpvObject::eventHandler()
 
             if (strcmp(prop->name, "time-pos") == 0) {
                 if (prop->format == MPV_FORMAT_DOUBLE) {
-                    emit positionChanged();
+                    Q_EMIT positionChanged();
                 }
             } else if (strcmp(prop->name, "media-title") == 0) {
                 if (prop->format == MPV_FORMAT_STRING) {
-                    emit mediaTitleChanged();
+                    Q_EMIT mediaTitleChanged();
                 }
             } else if (strcmp(prop->name, "time-remaining") == 0) {
                 if (prop->format == MPV_FORMAT_DOUBLE) {
-                    emit remainingChanged();
+                    Q_EMIT remainingChanged();
                 }
             } else if (strcmp(prop->name, "duration") == 0) {
                 if (prop->format == MPV_FORMAT_DOUBLE) {
-                    emit durationChanged();
+                    Q_EMIT durationChanged();
                 }
             } else if (strcmp(prop->name, "volume") == 0) {
                 if (prop->format == MPV_FORMAT_INT64) {
-                    emit volumeChanged();
+                    Q_EMIT volumeChanged();
                 }
             } else if (strcmp(prop->name, "pause") == 0) {
                 if (prop->format == MPV_FORMAT_FLAG) {
-                    emit pauseChanged();
+                    Q_EMIT pauseChanged();
                 }
             } else if (strcmp(prop->name, "chapter") == 0) {
                 if (prop->format == MPV_FORMAT_INT64) {
-                    emit chapterChanged();
+                    Q_EMIT chapterChanged();
                 }
             } else if (strcmp(prop->name, "aid") == 0) {
                 if (prop->format == MPV_FORMAT_INT64) {
-                    emit audioIdChanged();
+                    Q_EMIT audioIdChanged();
                 }
             } else if (strcmp(prop->name, "sid") == 0) {
                 if (prop->format == MPV_FORMAT_INT64) {
-                    emit subtitleIdChanged();
+                    Q_EMIT subtitleIdChanged();
                 }
             } else if (strcmp(prop->name, "secondary-sid") == 0) {
                 if (prop->format == MPV_FORMAT_INT64) {
-                    emit secondarySubtitleIdChanged();
+                    Q_EMIT secondarySubtitleIdChanged();
                 }
             } else if (strcmp(prop->name, "contrast") == 0) {
                 if (prop->format == MPV_FORMAT_INT64) {
-                    emit contrastChanged();
+                    Q_EMIT contrastChanged();
                 }
             } else if (strcmp(prop->name, "brightness") == 0) {
                 if (prop->format == MPV_FORMAT_INT64) {
-                    emit brightnessChanged();
+                    Q_EMIT brightnessChanged();
                 }
             } else if (strcmp(prop->name, "gamma") == 0) {
                 if (prop->format == MPV_FORMAT_INT64) {
-                    emit gammaChanged();
+                    Q_EMIT gammaChanged();
                 }
             } else if (strcmp(prop->name, "saturation") == 0) {
                 if (prop->format == MPV_FORMAT_INT64) {
-                    emit saturationChanged();
+                    Q_EMIT saturationChanged();
                 }
             } else if (strcmp(prop->name, "track-list") == 0) {
                 if (prop->format == MPV_FORMAT_NODE) {
@@ -543,8 +561,8 @@ void MpvObject::loadTracks()
     m_subtitleTracksModel->setTracks(m_subtitleTracks);
     m_audioTracksModel->setTracks(m_audioTracks);
 
-    emit audioTracksModelChanged();
-    emit subtitleTracksModelChanged();
+    Q_EMIT audioTracksModelChanged();
+    Q_EMIT subtitleTracksModelChanged();
 }
 
 TracksModel *MpvObject::subtitleTracksModel() const
@@ -563,7 +581,7 @@ void MpvObject::getYouTubePlaylist(const QString &path)
 
     // use youtube-dl to get the required playlist info as json
     auto ytdlProcess = new QProcess();
-    ytdlProcess->setProgram("youtube-dl");
+    ytdlProcess->setProgram(Application::youtubeDlExecutable());
     ytdlProcess->setArguments(QStringList() << "-J" << "--flat-playlist" << path);
     ytdlProcess->start();
 
@@ -597,7 +615,7 @@ void MpvObject::getYouTubePlaylist(const QString &path)
         // save playlist to disk
         m_playlistModel->setPlayList(m_playList);
 
-        emit youtubePlaylistLoaded();
+        Q_EMIT youtubePlaylistLoaded();
     });
 }
 
@@ -605,7 +623,7 @@ int MpvObject::setProperty(const QString &name, const QVariant &value, bool debu
 {
     auto result = mpv::qt::set_property(mpv, name, value);
     if (debug) {
-        DEBUG << mpv::qt::get_error(result);
+        qDebug() << name << mpv::qt::get_error(result);
     }
     return result;
 }
@@ -614,14 +632,18 @@ QVariant MpvObject::getProperty(const QString &name, bool debug)
 {
     auto result = mpv::qt::get_property(mpv, name);
     if (debug) {
-        DEBUG << mpv::qt::get_error(result);
+        qDebug() << name << mpv::qt::get_error(result);
     }
     return result;
 }
 
-QVariant MpvObject::command(const QVariant &params)
+QVariant MpvObject::command(const QVariant &params, bool debug)
 {
-    return mpv::qt::command(mpv, params);
+    auto result = mpv::qt::command(mpv, params);
+    if (debug) {
+        qDebug() << mpv::qt::get_error(result);
+    }
+    return result;
 }
 
 void MpvObject::saveTimePosition()
@@ -637,8 +659,8 @@ void MpvObject::saveTimePosition()
 
     auto hash = md5(getProperty("path").toString());
     auto timePosition = getProperty("time-pos");
-    auto configPath = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
-    KConfig *config = new KConfig(configPath.append("/georgefb/watch-later/").append(hash));
+    auto configPath = Global::instance()->appConfigDirPath();
+    KConfig *config = new KConfig(configPath.append("/watch-later/").append(hash));
     config->group("").writeEntry("TimePosition", timePosition);
     config->sync();
 }
@@ -657,8 +679,8 @@ double MpvObject::loadTimePosition()
     }
 
     auto hash = md5(getProperty("path").toString());
-    auto configPath = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
-    KConfig *config = new KConfig(configPath.append("/georgefb/watch-later/").append(hash));
+    auto configPath = Global::instance()->appConfigDirPath();
+    KConfig *config = new KConfig(configPath.append("/watch-later/").append(hash));
     int position = config->group("").readEntry("TimePosition", QString::number(0)).toDouble();
 
     return position;
@@ -667,14 +689,20 @@ double MpvObject::loadTimePosition()
 void MpvObject::resetTimePosition()
 {
     auto hash = md5(getProperty("path").toString());
-    auto configPath = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
-    QFile f(configPath.append("/georgefb/watch-later/").append(hash));
+    auto configPath = Global::instance()->appConfigDirPath();
+    QFile f(configPath.append("/watch-later/").append(hash));
 
     if (f.exists()) {
         f.remove();
     }
 }
 
+void MpvObject::userCommand(const QString &commandString)
+{
+    QStringList args = KShell::splitArgs(commandString);
+    command(args);
+}
+
 QString MpvObject::md5(const QString &str)
 {
     auto md5 = QCryptographicHash::hash((str.toUtf8()), QCryptographicHash::Md5);
