@@ -1,11 +1,13 @@
+/*
+ * SPDX-FileCopyrightText: 2021 George Florea Bănuș <georgefb899@gmail.com>
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
+ */
+
 #ifndef THUMBNAILIMAGEPROVIDER_H
 #define THUMBNAILIMAGEPROVIDER_H
 
 #include <QQuickAsyncImageProvider>
-#include <QDir>
-
-class KFileItem;
-class KJob;
 
 class ThumbnailImageProvider : public QQuickAsyncImageProvider
 {
@@ -17,14 +19,13 @@ public:
 
 class ThumbnailResponse : public QQuickImageResponse
 {
-    public:
-        ThumbnailResponse(const QString &id, const QSize &requestedSize);
+public:
+    ThumbnailResponse(const QString &id, const QSize &requestedSize);
 
-        QQuickTextureFactory *textureFactory() const override;
+    QQuickTextureFactory *textureFactory() const override;
+    void getPreview(const QString &id, const QSize &requestedSize);
 
-        QImage m_image;
-        QQuickTextureFactory *m_texture {nullptr};
-        void getPreview(const QString &id, const QSize &requestedSize);
+    QQuickTextureFactory *m_texture {nullptr};
 };
 
 #endif // THUMBNAILIMAGEPROVIDER_H
