@@ -14,19 +14,25 @@ class Worker : public QObject
 {
     Q_OBJECT
 public:
-    Worker() = default;
-    ~Worker() = default;
-
     static Worker* instance();
 
 signals:
     void metaDataReady(int index, KFileMetaData::PropertyMap metadata);
+    void thumbnailSuccess(const QImage &image);
+    void thumbnailFail();
 
 public slots:
     void getMetaData(int index, const QString &path);
+    void makePlaylistThumbnail(const QString &id, int width);
 
 private:
-    static Worker *sm_worker;
+    Worker() = default;
+    ~Worker() = default;
+
+    Worker(const Worker &) = delete;
+    Worker &operator=(const Worker &) = delete;
+    Worker(Worker &&) = delete;
+    Worker &operator=(Worker &&) = delete;
 };
 
 #endif // WORKER_H
